﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace SegmentDemo
{
    /// <summary>
    /// 线段装饰器
    /// </summary>
    public class SegmentLineDecorator : ContentControl
    {
        /// <summary>
        /// 线尺寸的依赖属性
        /// </summary>
        public static readonly DependencyProperty LineSizeProperty = DependencyProperty.Register(
            "LineSize", typeof(Size), typeof(SegmentLineDecorator), new FrameworkPropertyMetadata(new Size(10d, 10d), FrameworkPropertyMetadataOptions.AffectsRender));

        /// <summary>
        /// 线尺寸
        /// </summary>
        public Size LineSize
        {
            get { return (Size)GetValue(LineSizeProperty); }
            set { SetValue(LineSizeProperty, value); }
        }

        /// <summary>Draws the contents of a <see cref="T:System.Windows.Media.DrawingContext" /> object during the render pass of a <see cref="T:System.Windows.Controls.Border" />. </summary>
        /// <param name="dc">The <see cref="T:System.Windows.Media.DrawingContext" /> that defines the object to be drawn.</param>
        protected override void OnRender(DrawingContext dc)
        {
            Thickness thickness = Except(Union(new Thickness(), this.Margin), this.Padding);
            var segmentPanel = GetDescendantPanel(this, ref thickness);
            if (segmentPanel != null)
            {
                DrawSegment(segmentPanel, dc, thickness);
            }
            else
            {
                base.OnRender(dc); //找不到可装饰的对象,退化为普通Border
            }
        }

        private SegmentsPanel GetDescendantPanel(DependencyObject element, ref Thickness thickness)
        {
            if (element is FrameworkElement)
            {
                Assign(ref thickness, Union(thickness, (element as FrameworkElement).Margin));
            }

            if (element is Control)
            {
                Assign(ref thickness, Union(thickness, (element as Control).Padding));
            }

            if (element is SegmentsPanel)
            {
                return element as SegmentsPanel;
            }

            int childCount = VisualTreeHelper.GetChildrenCount(element);
            if (childCount != 1)
            {
                return null;
            }

            return GetDescendantPanel(VisualTreeHelper.GetChild(element, 0), ref thickness);
        }

        private void Assign(ref Thickness thickness1, Thickness thickness2)
        {
            thickness1.Left = thickness2.Left;
            thickness1.Top = thickness2.Top;
            thickness1.Right = thickness2.Right;
            thickness1.Bottom = thickness2.Bottom;
        }

        private Thickness Union(Thickness thickness1, Thickness thickness2)
        {
            double left = thickness1.Left + thickness2.Left;
            double top = thickness1.Top + thickness2.Top;
            double right = thickness1.Right + thickness2.Right;
            double bottom = thickness1.Bottom + thickness2.Bottom;

            return new Thickness(left, top, right, bottom);
        }

        private Thickness Except(Thickness thickness1, Thickness thickness2)
        {
            double left = thickness1.Left - thickness2.Left;
            double top = thickness1.Top - thickness2.Top;
            double right = thickness1.Right - thickness2.Right;
            double bottom = thickness1.Bottom - thickness2.Bottom;

            return new Thickness(left, top, right, bottom);
        }

        private void DrawSegment(SegmentsPanel segmentsPanel, DrawingContext dc, Thickness margin)
        {
            int visibleChildCount = CountVisibleChild(segmentsPanel);
            if (visibleChildCount > 1)
            {
                Size renderSize = segmentsPanel.RenderSize;
                List<double> positions = new List<double>();

                Func<Size, double> sizeValueFunc;
                if (segmentsPanel.Orientation == Orientation.Horizontal)
                {
                    sizeValueFunc = s => s.Width;
                }
                else
                {
                    sizeValueFunc = s => s.Height;
                }

                int firstVisible = 0;
                while (segmentsPanel.Children[firstVisible].Visibility == Visibility.Collapsed)
                {
                    firstVisible++;
                }

                UIElement firstChild = segmentsPanel.Children[firstVisible];
                positions.Add(sizeValueFunc(firstChild.DesiredSize) / 2d);

                int lastVisible = visibleChildCount - 1;
                while (segmentsPanel.Children[lastVisible].Visibility == Visibility.Collapsed)
                {
                    lastVisible--;
                }

                UIElement lastChild = segmentsPanel.Children[lastVisible];

                int ordinaryChildCount = visibleChildCount - 2;
                if (ordinaryChildCount > 0)
                {
                    double uniformWidth = (renderSize.Width - firstChild.DesiredSize.Width / 2.0 - lastChild.DesiredSize.Width / 2.0) / (ordinaryChildCount + 1);
                    double uniformHeight = (renderSize.Height - firstChild.DesiredSize.Height / 2.0 - lastChild.DesiredSize.Height / 2.0) / (ordinaryChildCount + 1);
                    Size uniformSize = new Size(uniformWidth, uniformHeight);

                    int visible = 0;
                    for (int i = firstVisible + 1; i < lastVisible; i++)
                    {
                        UIElement child = segmentsPanel.Children[i];
                        if (child.Visibility == Visibility.Collapsed)
                        {
                            continue;
                        }

                        visible++;
                        double position = sizeValueFunc(firstChild.DesiredSize) / 2d + sizeValueFunc(uniformSize) * visible;
                        positions.Add(position);
                    }
                }

                positions.Add(sizeValueFunc(renderSize) - sizeValueFunc(lastChild.DesiredSize) / 2d);

                for (int i = 0; i < positions.Count - 1; i++)
                {
                    if (segmentsPanel.Orientation == Orientation.Horizontal)
                    {
                        DrawHorizontal(dc, positions[i], positions[i + 1],
                            renderSize.Height, margin, this.Padding);
                    }
                    else
                    {
                        DrawVertical(dc, positions[i], positions[i + 1],
                            renderSize.Width, margin, this.Padding);
                    }
                }
            }
        }

        private int CountVisibleChild(SegmentsPanel segmentsPanel)
        {
            return segmentsPanel.Children.Cast<UIElement>().Count(element => element.Visibility != Visibility.Collapsed);
        }

        private void DrawHorizontal(DrawingContext dc, double left, double right, double height, Thickness margin, Thickness padding)
        {
            double y = 0;
            double lineHeight = LineSize.Height;

            double yAvailable = height - padding.Top - padding.Bottom;
            if (yAvailable <= 0)
            {
                lineHeight = Math.Min(lineHeight, Height);
            }
            else
            {
                switch (VerticalContentAlignment)
                {
                    case VerticalAlignment.Top:
                        y = padding.Top;
                        break;
                    case VerticalAlignment.Center:
                        y = yAvailable / 2d + padding.Top - lineHeight / 2d;
                        break;
                    case VerticalAlignment.Bottom:
                        y = height - padding.Bottom;
                        break;
                    case VerticalAlignment.Stretch:
                        y = padding.Top;
                        lineHeight = yAvailable;
                        break;
                }
            }
            var size = new Size(right - left - padding.Left - padding.Right, lineHeight);
            var rect = new Rect(new Point(margin.Left + left + padding.Left, margin.Top + y), size);
            dc.DrawRectangle(Background, new Pen(Brushes.Transparent, 0), rect);

            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Top), new Point(rect.X, rect.Y), new Point(rect.X + rect.Width, rect.Y));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Right), new Point(rect.X + rect.Width, rect.Y), new Point(rect.X + rect.Width, rect.Y + rect.Height));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Bottom), new Point(rect.X + rect.Width, rect.Y + rect.Height), new Point(rect.X, rect.Y + rect.Height));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Left), new Point(rect.X, rect.Y + rect.Height), new Point(rect.X, rect.Y));
        }

        private void DrawVertical(DrawingContext dc, double top, double bottom, double width, Thickness margin, Thickness padding)
        {
            double x = 0;
            double lineWidth = LineSize.Width;

            double xAvailable = width - padding.Left - padding.Right;
            if (xAvailable <= 0)
            {
                lineWidth = Math.Min(lineWidth, Width);
            }
            else
            {
                switch (HorizontalContentAlignment)
                {
                    case HorizontalAlignment.Left:
                        x = padding.Left;
                        break;
                    case HorizontalAlignment.Center:
                        x = xAvailable / 2d + padding.Left - lineWidth / 2d;
                        break;
                    case HorizontalAlignment.Right:
                        x = width - padding.Right;
                        break;
                    case HorizontalAlignment.Stretch:
                        x = padding.Left;
                        lineWidth = xAvailable;
                        break;
                }
            }
            var size = new Size(lineWidth, bottom - top - padding.Top - padding.Bottom);
            var rect = new Rect(new Point(margin.Left + x, margin.Top + top + padding.Top), size);
            dc.DrawRectangle(Background, new Pen(Brushes.Transparent, 0), rect);

            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Top), new Point(rect.X, rect.Y), new Point(rect.X + rect.Width, rect.Y));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Right), new Point(rect.X + rect.Width, rect.Y), new Point(rect.X + rect.Width, rect.Y + rect.Width));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Bottom), new Point(rect.X + rect.Width, rect.Y + rect.Width), new Point(rect.X, rect.Y + rect.Width));
            dc.DrawLine(new Pen(BorderBrush, BorderThickness.Left), new Point(rect.X, rect.Y + rect.Width), new Point(rect.X, rect.Y));
        }
    }
}
