﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace SegmentDemo
{
    /// <summary>
    /// 类似线段的布局面板,即在最左侧朝右布局，最右侧朝左布局，中间点居中布局
    /// </summary>
    public class SegmentsPanel : Panel
    {
        /// <summary>
        /// 可见子元素个数
        /// </summary>
        private int _visibleChildCount;

        /// <summary>
        /// 朝向的依赖属性
        /// </summary>
        public static readonly DependencyProperty OrientationProperty = DependencyProperty.Register(
            "Orientation", typeof(Orientation), typeof(SegmentsPanel),
            new FrameworkPropertyMetadata(Orientation.Horizontal, FrameworkPropertyMetadataOptions.AffectsMeasure));

        /// <summary>
        /// 朝向
        /// </summary>
        public Orientation Orientation
        {
            get { return (Orientation)GetValue(OrientationProperty); }
            set { SetValue(OrientationProperty, value); }
        }

        protected override Size MeasureOverride(Size constraint)
        {
            _visibleChildCount = this.CountVisibleChild();

            if (_visibleChildCount == 0)
            {
                return new Size(0, 0);
            }

            double width = 0;
            double height = 0;

            Size availableSize = new Size(constraint.Width / _visibleChildCount, constraint.Height);

            if (Orientation == Orientation.Vertical)
            {
                availableSize = new Size(constraint.Width, constraint.Height / _visibleChildCount);
            }

            foreach (UIElement child in Children)
            {
                child.Measure(availableSize);
                Size desiredSize = child.DesiredSize;

                if (Orientation == Orientation.Horizontal)
                {
                    width += desiredSize.Width;
                    height = Math.Max(height, desiredSize.Height);
                }
                else
                {
                    width = Math.Max(width, desiredSize.Width);
                    height += desiredSize.Height;
                }
            }

            return new Size(width, height);
        }

        protected override Size ArrangeOverride(Size arrangeSize)
        {
            if (_visibleChildCount == 0)
            {
                return arrangeSize;
            }

            int firstVisible = 0;
            while (InternalChildren[firstVisible].Visibility == Visibility.Collapsed)
            {
                firstVisible++;
            }

            UIElement firstChild = this.InternalChildren[firstVisible];
            if (Orientation == Orientation.Horizontal)
            {
                this.ArrangeChildHorizontal(firstChild, arrangeSize.Height, 0);
            }
            else
            {
                this.ArrangeChildVertical(firstChild, arrangeSize.Width, 0);
            }

            int lastVisible = _visibleChildCount - 1;
            while (InternalChildren[lastVisible].Visibility == Visibility.Collapsed)
            {
                lastVisible--;
            }

            if (lastVisible <= firstVisible)
            {
                return arrangeSize;
            }

            UIElement lastChild = this.InternalChildren[lastVisible];
            if (Orientation == Orientation.Horizontal)
            {
                this.ArrangeChildHorizontal(lastChild, arrangeSize.Height, arrangeSize.Width - lastChild.DesiredSize.Width);
            }
            else
            {
                this.ArrangeChildVertical(lastChild, arrangeSize.Width, arrangeSize.Height - lastChild.DesiredSize.Height);
            }

            int ordinaryChildCount = _visibleChildCount - 2;
            if (ordinaryChildCount > 0)
            {
                double uniformWidth = (arrangeSize.Width  - firstChild.DesiredSize.Width / 2.0 - lastChild.DesiredSize.Width / 2.0) / (ordinaryChildCount + 1);
                double uniformHeight = (arrangeSize.Height - firstChild.DesiredSize.Height / 2.0 - lastChild.DesiredSize.Height / 2.0) / (ordinaryChildCount + 1);

                int visible = 0;
                for (int i = firstVisible + 1; i < lastVisible; i++)
                {
                    UIElement child = this.InternalChildren[i];
                    if (child.Visibility == Visibility.Collapsed)
                    {
                        continue;
                    }

                    visible++;

                    if (Orientation == Orientation.Horizontal)
                    {
                        double x = firstChild.DesiredSize.Width / 2.0 + uniformWidth * visible - child.DesiredSize.Width / 2.0;
                        this.ArrangeChildHorizontal(child, arrangeSize.Height, x);
                    }
                    else
                    {
                        double y = firstChild.DesiredSize.Height / 2.0 + uniformHeight * visible - child.DesiredSize.Height / 2.0;
                        this.ArrangeChildVertical(child, arrangeSize.Width, y);
                    }
                }
            }

            return arrangeSize;
        }

        /// <summary>
        /// 统计可见的子元素数
        /// </summary>
        /// <returns>可见子元素数</returns>
        private int CountVisibleChild()
        {
            return this.InternalChildren.Cast<UIElement>().Count(element => element.Visibility != Visibility.Collapsed);
        }

        /// <summary>
        /// 在水平方向安排子元素
        /// </summary>
        /// <param name="child">子元素</param>
        /// <param name="height">可用的高度</param>
        /// <param name="x">水平方向起始坐标</param>
        private void ArrangeChildHorizontal(UIElement child, double height, double x)
        {
            child.Arrange(new Rect(new Point(x, 0), new Size(child.DesiredSize.Width, height)));
        }

        /// <summary>
        /// 在竖直方向安排子元素
        /// </summary>
        /// <param name="child">子元素</param>
        /// <param name="width">可用的宽度</param>
        /// <param name="y">竖直方向起始坐标</param>
        private void ArrangeChildVertical(UIElement child, double width, double y)
        {
            child.Arrange(new Rect(new Point(0, y), new Size(width, child.DesiredSize.Height)));
        }
    }
}